/*
 *
 * Copyright (C) 2010 Michal Malek <michalm@jabster.pl>
 *
 * This file is part of the K3b project.
 * Copyright (C) 1998-2008 Sebastian Trueg <trueg@k3b.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 */

#ifndef _K3B_SOX_ENCODER_DEFAULTS_H_
#define _K3B_SOX_ENCODER_DEFAULTS_H_

const bool DEFAULT_MANUAL_SETTINGS = false;
const int DEFAULT_CHANNELS = 2;
const int DEFAULT_SAMPLE_RATE = 44100;
const char DEFAULT_DATA_ENCODING[] = "signed";
const int DEFAULT_DATA_SIZE = 16;

#endif // _K3B_SOX_ENCODER_DEFAULTS_H_
